//------------------------------------------------------------------------------
// File: PlayCap.cpp
//
// Desc: DirectShow sample code - a very basic application using Capture
//       Devices.  It creates a window and uses the first available Capture
//       Device to render and preview video capture data.
//
//------------------------------------------------------------------------------


#include <atlbase.h>
#include <windows.h>
#include <dshow.h>
#include <stdio.h>
#include <qedit.h>
#include <initguid.h> //Johnson, '00/03/05
#include "vfw.h"
#include "mtype.h"

#include "dshowutil.h"
#include "playcap.h"
#include "resource.h" //Johnson, '00/03/05
#include "PvCallBack.h"
#include "PVProp.h"

// An application can advertise the existence of its filter graph
// by registering the graph with a global Running Object Table (ROT).
// The GraphEdit application can detect and remotely view the running
// filter graph, allowing you to 'spy' on the graph with GraphEdit.
//
// To enable registration in this sample, define REGISTER_FILTERGRAPH.
//
#define REGISTER_FILTERGRAPH

//Video Format VF_NTSC/VF_PAL
#define VF_NTSC
//#define VF_PAL

//ProVideo Capture Card Number 0 ~ 7
#define CARD_NUM	0


//Capture size 80x60, 160x120, 320x240, 640x480...
//If you set 640x480, you can't enable all video source.
#define CAP_SIZE_640x480	1

#if CAP_SIZE_640x480
	#define CAP_SRC_W	640	//Source image = 640x240
	#define CAP_SRC_H	240
	#define CAP_DUP_W	640
	#define CAP_DUP_H	480
#else
	#define CAP_SRC_W	320	//Source image = 320x240
	#define CAP_SRC_H	240
	#define CAP_DUP_W	320
	#define CAP_DUP_H	240
#endif

#define IMAGE_BUFFER_SIZE	(CAP_DUP_W * CAP_DUP_H * 2)	//Width * Height * RGB15
#define ID_TIMER_WDT	100

#if CARD_NUM == 0
const char szDeviceName[] = {"BT878 Capture 0"};	//Device driver name
#elif CARD_NUM == 1
const char szDeviceName[] = {"BT878 Capture 1"};	//Device driver name
#elif CARD_NUM == 2
const char szDeviceName[] = {"BT878 Capture 2"};	//Device driver name
#elif CARD_NUM == 3
const char szDeviceName[] = {"BT878 Capture 3"};	//Device driver name
#elif CARD_NUM == 4
const char szDeviceName[] = {"BT878 Capture 4"};	//Device driver name
#elif CARD_NUM == 5
const char szDeviceName[] = {"BT878 Capture 5"};	//Device driver name
#elif CARD_NUM == 6
const char szDeviceName[] = {"BT878 Capture 6"};	//Device driver name
#else CARD_NUM == 7
const char szDeviceName[] = {"BT878 Capture 7"};	//Device driver name
#endif

//bool PV_SetBT878LocalReg(int iDevNum, DWORD dwReg, DWORD dwData);

//bool PV_I2CWrite3Byte(int iDevNum, BYTE byAddr, BYTE bySub, BYTE byData);

//
// Global data
//
HWND ghApp = 0;
HINSTANCE g_hInstance = 0;

DWORD g_dwGraphRegister=0;

IMediaControl * g_pMC = NULL;
IMediaEventEx * g_pME = NULL;
IGraphBuilder * g_pGraph = NULL;
ICaptureGraphBuilder2 * g_pCapture = NULL;
PLAYSTATE g_psCurrent = Stopped;

IBaseFilter     *g_pBF = NULL;
IAMAnalogVideoDecoder *g_pVdoDec = NULL;
IAMStreamConfig	*g_pVSC = NULL;      // for video capture pin

IBaseFilter     *g_pPvCallBack = NULL;	// Johnson,'02/01/14
IPvCallBackRender	*g_pCallBackProp = NULL;	//Johnson,'02/01/14

BYTE		g_byHasMMX = 0;
BYTE	g_bySelectDecoderNum;

static	BYTE	byBuffer[IMAGE_BUFFER_SIZE]; //Width * Height * RGB15

static	HDRAWDIB	hDrawDib;
static	BITMAPINFOHEADER BmpInfHdr;

HWND	g_VdoWnd_0, g_VdoWnd_1;

static BOOL hasMMX()
{
	BOOL found;

	_asm
	{
		mov found, 0
		mov eax, 1
		CPUID
		test edx, 0x800000
		jz MMX_notfound
		mov found, 1
MMX_notfound:
	}
	return found;
}

static void memcpyMMX(void *dst, void *src, int count)
{
	count >>= 3;
	_asm
	{
		mov esi, src
		mov edi, dst
		mov ecx, count
__L2:
		movq mm0, [esi]
		movq [edi], mm0
		add esi, 8
		add edi, 8
		dec ecx
		jnz __L2
		emms
	}
}

//Draw video here.
static HRESULT CALLBACK VideoCallBackProc(BYTE *pVdoBuffer, BYTE *pVdoSrc, LPVOID lpParam)
{
	static	RECT	rc;
	static	HDC		hDC;
	int		j;

	j=0;
	if(g_byHasMMX == 1)
	{
#if CAP_SIZE_640x480
		int i;
		for(i=0; i<CAP_SRC_H; i++)
		{
			memcpyMMX(&byBuffer[j*(CAP_SRC_W*2)], &pVdoBuffer[i*(CAP_SRC_W*2)], CAP_SRC_W*2); //2 byte = RGB555
			j++;	//Duplicate source image height 240 to 480
			memcpyMMX(&byBuffer[j*(CAP_SRC_W*2)], &pVdoBuffer[i*(CAP_SRC_W*2)], CAP_SRC_W*2);
			j++;
		}
#else
		memcpyMMX(byBuffer, pVdoBuffer, IMAGE_BUFFER_SIZE);
#endif	//CAP_SIZE_640x480  1
	}
	else
	{
#if CAP_SIZE_640x480
		int i;
		for(i=0; i<CAP_SRC_H; i++)
		{
			CopyMemory(&byBuffer[j*(CAP_SRC_W*2)], &pVdoBuffer[i*(CAP_SRC_W*2)], CAP_SRC_W*2);
			j++;	//Duplicate source image height 240 to 480
			CopyMemory(&byBuffer[j*(CAP_SRC_W*2)], &pVdoBuffer[i*(CAP_SRC_W*2)], CAP_SRC_W*2);
			j++;
		}
#else
		CopyMemory(byBuffer, pVdoBuffer, IMAGE_BUFFER_SIZE);
#endif	//CAP_SIZE_640x480  1
	}

	if(*pVdoSrc == 0)
	{
		GetClientRect(g_VdoWnd_0, &rc);
		hDC = GetDC(g_VdoWnd_0);
		DrawDibBegin(hDrawDib, hDC, 0, 0, &BmpInfHdr, 0, 0, 0x0200);
		DrawDibDraw(hDrawDib, hDC, rc.left, rc.top, rc.right, rc.bottom
					, &BmpInfHdr, byBuffer, 0, 0,
					CAP_DUP_W, CAP_DUP_H, 0x0200);
		ReleaseDC(g_VdoWnd_0, hDC);
	}
	else if(*pVdoSrc == 1)
	{
		GetClientRect(g_VdoWnd_1, &rc);
		hDC = GetDC(g_VdoWnd_1);
		DrawDibBegin(hDrawDib, hDC, 0, 0, &BmpInfHdr, 0, 0, 0x0200);
		DrawDibDraw(hDrawDib, hDC, rc.left, rc.top, rc.right, rc.bottom
					, &BmpInfHdr, byBuffer, 0, 0,
					CAP_DUP_W, CAP_DUP_H, 0x0200);
		ReleaseDC(g_VdoWnd_1, hDC);
	}

	return S_OK;
}

void SetCaptureSize()
{
	if(g_pVSC == NULL)
		return;

	HRESULT hr;
	AM_MEDIA_TYPE *pmt;

	// get format being used NOW
	hr = g_pVSC->GetFormat(&pmt);
	// DV capture does not use a VIDEOINFOHEADER
	if(SUCCEEDED(hr) )
	{
		if (pmt->formattype == FORMAT_VideoInfo)
		{
			pmt->subtype = MEDIASUBTYPE_RGB555;
			HEADER(pmt->pbFormat)->biWidth = CAP_SRC_W;
			HEADER(pmt->pbFormat)->biHeight = CAP_SRC_H;
			g_pVSC->SetFormat(pmt);
		}
		DeleteMediaType(pmt);
	}
}

BOOL SetVideoFormat(long lFormat)
{
	HRESULT hr;
	long	lAvailableTVFormats = 0;

	if(g_pVdoDec == NULL)
		return FALSE;

	hr = g_pVdoDec->get_AvailableTVFormats(&lAvailableTVFormats);
	if( FAILED( hr ) )
		return FALSE;

	if( (lFormat & lAvailableTVFormats) == 0)
		return FALSE;  //Not support this format.

	hr = g_pVdoDec->put_TVFormat(lFormat);


	if(hr == S_OK)
		return TRUE;
	else
		return FALSE;
}

HRESULT ConnectGrabberPin(IBaseFilter *pCapFilter)	//Johnson
{
    HRESULT hr;

	if(g_pPvCallBack == NULL || pCapFilter == NULL)
	{
		Msg(TEXT( "Could not find filter pointer."));
		return S_FALSE;
	}

	// find the two pins and connect them
	IPin * pCapOut = GetOutPin( pCapFilter, 0 );//0:Capture, 1:Preview
	IPin * pGrabIn = GetInPin( g_pPvCallBack, 0 );
    hr = g_pGraph->Connect( pCapOut, pGrabIn );
	if( FAILED( hr ) )
	{
        Msg(TEXT( "Could not connect capture pin #0 to grabber.\r\n"
               "Is the capture device being used by another application?" ));
        return hr;
	}

	return hr;
}

HRESULT CaptureVideo()
{
    HRESULT hr;
    IBaseFilter *pSrcFilter=NULL;

	g_byHasMMX = (BYTE)hasMMX();

    // Get DirectShow interfaces
    hr = GetInterfaces();
    if (FAILED(hr))
    {
        Msg(TEXT("Failed to get video interfaces or not found DX8!  hr=0x%x"), hr);
        return hr;
    }

	PV_InitProperty();
	PV_SetVideoSource(CARD_NUM, POLLING_ALL);

    // Attach the filter graph to the capture graph
    hr = g_pCapture->SetFiltergraph(g_pGraph);
    if (FAILED(hr))
    {
        Msg(TEXT("Failed to set capture filter graph!  hr=0x%x"), hr);
        return hr;
    }

    // Use the system device enumerator and class enumerator to find
    // a video capture/preview device, such as a desktop USB video camera.
    hr = FindCaptureDevice(&pSrcFilter);
    if (FAILED(hr))
    {
        // Don't display a message because FindCaptureDevice will handle it
        return hr;
    }

	hr = g_pCapture->FindInterface(&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Video, pSrcFilter,
							IID_IAMAnalogVideoDecoder, (void **)&g_pVdoDec);
    if (FAILED(hr))
        return hr;

    hr = g_pCapture->FindInterface(&PIN_CATEGORY_CAPTURE, &MEDIATYPE_Video, pSrcFilter,
				IID_IAMStreamConfig, (void **)&g_pVSC);
    if (FAILED(hr))
        return hr;

    // Add Capture filter to our graph.
    hr = g_pGraph->AddFilter(pSrcFilter, L"Video Capture");
    if (FAILED(hr))
    {
        Msg(TEXT("Couldn't add capture filter to graph!  hr=0x%x"), hr);
        pSrcFilter->Release();
        return hr;
    }
#ifdef VF_NTSC
	SetVideoFormat((long)AnalogVideo_NTSC_M);
#else
	SetVideoFormat((long)AnalogVideo_PAL_B);
#endif

	SetCaptureSize(); //Johnson, '01/03/20
	ConnectGrabberPin(pSrcFilter);	//Johnson
    if (FAILED(hr))
    {
		Msg(TEXT("Couldn't found XBar."));
	}
    // Now that the filter has been added to the graph and we have
    // rendered its stream, we can release this reference to the filter.
    pSrcFilter->Release();

    // Add our graph to the running object table, which will allow
    // the GraphEdit application to "spy" on our graph
#ifdef REGISTER_FILTERGRAPH
    hr = AddGraphToRot(g_pGraph, &g_dwGraphRegister);
    if (FAILED(hr))
    {
        Msg(TEXT("Failed to register filter graph with ROT!  hr=0x%x"), hr);
        g_dwGraphRegister = 0;
    }
#endif

	if(g_pCallBackProp)
		g_pCallBackProp->SetCallBackFunction(VideoCallBackProc, NULL);

    // Start previewing video data
    hr = g_pMC->Run();
    if (FAILED(hr))
    {
        Msg(TEXT("Couldn't run the graph!  hr=0x%x"), hr);
        return hr;
    }

	//When the WaitForCompletion method returns,
	//the Sample Grabber's buffer holds a copy of the video frame.
	//long evCode;
	//hr = g_pME->WaitForCompletion(INFINITE, &evCode); //In CallBack don't use it!

    // Remember current state
    g_psCurrent = Running;
        
    return S_OK;
}


HRESULT FindCaptureDevice(IBaseFilter ** ppSrcFilter)
{
    USES_CONVERSION;	//Johnson, Convert wide char to text.

    HRESULT hr;
    IBaseFilter * pSrc = NULL;
    //CComPtr <IMoniker> pMoniker =NULL;
    IMoniker *pMoniker = NULL;
    ULONG cFetched;

    // Create the system device enumerator
    CComPtr <ICreateDevEnum> pDevEnum =NULL;

    hr = CoCreateInstance (CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC,
        IID_ICreateDevEnum, (void ** ) &pDevEnum);
    if (FAILED(hr))
    {
        Msg(TEXT("Couldn't create system enumerator!  hr=0x%x"), hr);
        return hr;
    }

    // Create an enumerator for the video capture devices
    CComPtr <IEnumMoniker> pClassEnum = NULL;

    hr = pDevEnum->CreateClassEnumerator (CLSID_VideoInputDeviceCategory, &pClassEnum, 0);
    if (FAILED(hr))
    {
        Msg(TEXT("Couldn't create class enumerator!  hr=0x%x"), hr);
        return hr;
    }

    // If there are no enumerators for the requested type, then 
    // CreateClassEnumerator will succeed, but pClassEnum will be NULL.
    if (pClassEnum == NULL)
    {
        MessageBox(ghApp,TEXT("No video capture device was detected.\r\n\r\n")
                   TEXT("This sample requires a video capture device, such as a USB WebCam,\r\n")
                   TEXT("to be installed and working properly.  The sample will now close."),
                   TEXT("No Video Capture Hardware"), MB_OK | MB_ICONINFORMATION);
        return E_FAIL;
    }

    // Use the first video capture device on the device list.
    // Note that if the Next() call succeeds but there are no monikers,
    // it will return S_FALSE (which is not a failure).  Therefore, we
    // check that the return code is S_OK instead of using SUCCEEDED() macro.
    while(S_OK == (pClassEnum->Next (1, &pMoniker, &cFetched)))
    {
		IPropertyBag *pBag;	//Johnson
		hr = pMoniker->BindToStorage(0, 0, IID_IPropertyBag, (void **)&pBag);
		if(SUCCEEDED(hr))
		{
			VARIANT var;
			var.vt = VT_BSTR;
			hr = pBag->Read(L"FriendlyName", &var, NULL);
			char	szText[64];
			wsprintf(szText, "%s", W2T(var.bstrVal));
			if(memcmp( szText, szDeviceName, sizeof(szDeviceName) ) == 0)
			{
				SetWindowText(ghApp, szText );
				// Bind Moniker to a filter object
				hr = pMoniker->BindToObject(0,0,IID_IBaseFilter, (void**)&pSrc);
				if (FAILED(hr))
				{
					Msg(TEXT("Couldn't bind moniker to filter object!  hr=0x%x"), hr);
					return hr;
				}
			}
			SysFreeString(var.bstrVal);
			pBag->Release();
		}
		if(pMoniker)
		{
			pMoniker->Release();
			pMoniker = NULL;
		}
    }

	if(pSrc == NULL)
		return S_FALSE;

    // Copy the found filter pointer to the output parameter.
    // Do NOT Release() the reference, since it will still be used
    // by the calling function.
    *ppSrcFilter = pSrc;

    return hr;
}


HRESULT GetInterfaces(void)
{
    HRESULT hr;

    // Create the filter graph
    hr = CoCreateInstance (CLSID_FilterGraph, NULL, CLSCTX_INPROC,
        IID_IGraphBuilder, (void **) &g_pGraph);
    if (FAILED(hr))
        return hr;

    // Create the capture graph builder
    hr = CoCreateInstance (CLSID_CaptureGraphBuilder2 , NULL, CLSCTX_INPROC,
        IID_ICaptureGraphBuilder2, (void **) &g_pCapture);
    if (FAILED(hr))
        return hr;

	// Create PV_CallBackRender
	hr = CoCreateInstance(CLSID_PvCallBackRender, NULL, CLSCTX_INPROC,
	    IID_IBaseFilter, (LPVOID *)&g_pPvCallBack);
    if (FAILED(hr))
        return hr;
	hr = g_pGraph->AddFilter(g_pPvCallBack, L"PV_CallBackRender");
    if (FAILED(hr))
        return hr;
	hr = g_pPvCallBack->QueryInterface(IID_IPvCallBackRender, (LPVOID *)&g_pCallBackProp);
    if (FAILED(hr))
        return hr;

    // Obtain interfaces for media control and Video Window
    hr = g_pGraph->QueryInterface(IID_IMediaControl,(LPVOID *) &g_pMC);
    if (FAILED(hr))
        return hr;

    hr = g_pGraph->QueryInterface(IID_IMediaEvent, (LPVOID *) &g_pME);
    if (FAILED(hr))
        return hr;

    // Set the window handle used to process graph events
    hr = g_pME->SetNotifyWindow((OAHWND)ghApp, WM_GRAPHNOTIFY, 0);

    return hr;
}


void CloseInterfaces(void)
{
    // Stop previewing data
    if (g_pMC)
        g_pMC->StopWhenReady();

    g_psCurrent = Stopped;

    // Stop receiving events
    if (g_pME)
        g_pME->SetNotifyWindow(NULL, WM_GRAPHNOTIFY, 0);


#ifdef REGISTER_FILTERGRAPH
    // Remove filter graph from the running object table   
    if (g_dwGraphRegister)
        RemoveGraphFromRot(g_dwGraphRegister);
#endif

	PV_UnInitProperty();

    // Release DirectShow interfaces
    SAFE_RELEASE(g_pPvCallBack);
    SAFE_RELEASE(g_pVSC);
    SAFE_RELEASE(g_pVdoDec);
    SAFE_RELEASE(g_pMC);
    SAFE_RELEASE(g_pME);
    SAFE_RELEASE(g_pBF);
    SAFE_RELEASE(g_pGraph);
    SAFE_RELEASE(g_pCapture);
}


HRESULT ChangePreviewState(int nShow)
{
    HRESULT hr;
    
    // If the media control interface isn't ready, don't call it
    if (!g_pMC)
        return S_OK;
    
    if (nShow)
    {
        if (g_psCurrent != Running)
        {
            // Start previewing video data
            hr = g_pMC->Run();
            g_psCurrent = Running;
        }
    }
    else
    {
        // Stop previewing video data
        hr = g_pMC->StopWhenReady();
        g_psCurrent = Stopped;
    }

    return hr;
}

void Msg(char *szFormat, ...)
{
    TCHAR szBuffer[512];

    va_list pArgs;
    va_start(pArgs, szFormat);
    vsprintf(szBuffer, szFormat, pArgs);
    va_end(pArgs);

    MessageBox(NULL, szBuffer, "PlayCap Message", MB_OK | MB_ICONERROR);
}


HRESULT HandleGraphEvent(void)
{
    LONG evCode, evParam1, evParam2;
    HRESULT hr=S_OK;

    while(SUCCEEDED(g_pME->GetEvent(&evCode, &evParam1, &evParam2, 0)))
    {
        //
        // Free event parameters to prevent memory leaks associated with
        // event parameter data.  While this application is not interested
        // in the received events, applications should always process them.
        //
        hr = g_pME->FreeEventParams(evCode, evParam1, evParam2);
        
        // Insert event processing code here, if desired
    }

    return hr;
}

static BYTE g_byCurrentSel;

static BOOL CALLBACK DlgProcAdjustColor (HWND hDlg, UINT uMsg, UINT wParam, LONG lParam)
{
	static HWND	hBrightness, hContrast;
	int		iScrollPos;

	switch (uMsg)
	{
		case WM_INITDIALOG:
			hBrightness = GetDlgItem(hDlg, IDC_BRIGHTNESS);
			hContrast = GetDlgItem(hDlg, IDC_CONTRAST);

			SCROLLINFO	sinf;
			sinf.cbSize = sizeof(sinf);
			sinf.fMask = SIF_RANGE | SIF_POS | SIF_PAGE;
			sinf.nMin = (int)0;
			sinf.nMax = (int)10000;
			sinf.nPage = 100;

			sinf.nPos = 5000;
			SetScrollInfo( hBrightness, SB_CTL, &sinf, FALSE );
			SetScrollInfo( hContrast, SB_CTL, &sinf, FALSE );
			PV_SetBrightness(CARD_NUM, g_bySelectDecoderNum, sinf.nPos);
			PV_SetContrast(CARD_NUM, g_bySelectDecoderNum, sinf.nPos);
			return TRUE;

		case WM_HSCROLL:
			iScrollPos = GetScrollPos((HWND)lParam, SB_CTL);
			switch ( (int) LOWORD(wParam))
			{
				case SB_LINERIGHT:
				case SB_PAGERIGHT:
					if(iScrollPos < 250)
						iScrollPos += 5;
					break;

				case SB_LINELEFT:
				case SB_PAGELEFT:
					if(iScrollPos > 5)
						iScrollPos -= 5;
					break;

				case SB_THUMBPOSITION:
				case SB_THUMBTRACK:
					iScrollPos = (int)HIWORD(wParam);
					break;
			}
			if((HWND)lParam == hBrightness)
				PV_SetBrightness(CARD_NUM, g_bySelectDecoderNum, iScrollPos);
			else if((HWND)lParam == hContrast)
				PV_SetContrast(CARD_NUM, g_bySelectDecoderNum, iScrollPos);

			SetScrollPos( (HWND)lParam, SB_CTL, iScrollPos, TRUE );
			break;

		case WM_COMMAND:
			switch (wParam)
			{
				case IDOK:
					EndDialog(hDlg, TRUE) ;
					return TRUE ;
				case IDCANCEL:
					EndDialog(hDlg, FALSE) ;
					return TRUE ;
			}
			break;

		default:
			return FALSE;
	}
	return FALSE;
}

static BOOL CALLBACK DlgProcTestInOutPin (HWND hDlg, UINT uMsg, UINT wParam, LONG lParam)
{
#define	GPIO_TOTAL_BIT	16
	static	HWND	hHexText;
	static	HWND	hPinCheck[GPIO_TOTAL_BIT];
	WORD	wData = 0;
	WORD	wMask = 0x0001;
	WORD	wResult = 0;
	char	szHexText[8];

	switch (uMsg)
	{
		int	i;
		case WM_INITDIALOG:
			PV_GpioInitialize(0, 0xff00);	//Bit 0~7 input, Bit 8~15 output, total 16 bit
			for(i=0; i<GPIO_TOTAL_BIT; i++)
			{
				SendDlgItemMessage(hDlg, (IDC_PIN_CHECK0 + i), BM_SETCHECK, 0, 0);
				hPinCheck[i] = GetDlgItem(hDlg, IDC_PIN_CHECK0 + i);
			}
			hHexText = GetDlgItem(hDlg, IDC_STATIC_HEX);
			SetWindowText(hHexText, "0x0000");
			PV_GpioSetData(0, 0x0000);
			return true;

		case WM_COMMAND:
			switch (HIWORD(wParam) )
			{
				case BN_CLICKED:
					switch (LOWORD(wParam))
					{
						case IDC_PIN_CHECK0:	//MSB
						case IDC_PIN_CHECK1:
						case IDC_PIN_CHECK2:
						case IDC_PIN_CHECK3:
						case IDC_PIN_CHECK4:
						case IDC_PIN_CHECK5:
						case IDC_PIN_CHECK6:
						case IDC_PIN_CHECK7:
						case IDC_PIN_CHECK8:
						case IDC_PIN_CHECK9:
						case IDC_PIN_CHECK10:
						case IDC_PIN_CHECK11:
						case IDC_PIN_CHECK12:
						case IDC_PIN_CHECK13:
						case IDC_PIN_CHECK14:
						case IDC_PIN_CHECK15:	//LSB
							wData = 0;
							for(i=0; i<GPIO_TOTAL_BIT; i++)
							{
								if(SendMessage(hPinCheck[i], BM_GETCHECK, NULL, NULL) == BST_CHECKED)
								{
									wData |= 0x8000;
								}
								if(i < (GPIO_TOTAL_BIT-1) )
									wData = wData >> 1;
							}
							wsprintf(szHexText, "0x%.4x", wData);
							SetWindowText(hHexText, szHexText);
							PV_GpioSetData(0, wData);
							break;
					}
					break;
			}

			switch (wParam)
			{
				case IDB_GET:
					PV_GpioGetData(0, &wData);
					wData &= 0xffff;
					wsprintf(szHexText, "0x%.4x", wData);
					SetWindowText(hHexText, szHexText);
					wMask = 0x0001;
					wResult = 0;
					for(i=0; i<GPIO_TOTAL_BIT; i++)
					{
						wResult = wMask & wData;
						SendMessage(hPinCheck[i], BM_SETCHECK, 
							(wResult==0) ? BST_UNCHECKED:BST_CHECKED, NULL);
						wMask = wMask << 1;
						wResult = 0;
					}
					break;

				case IDOK:
					EndDialog(hDlg, TRUE);
					return TRUE;
				case IDCANCEL:
					EndDialog(hDlg, FALSE);
					return TRUE;
			}
			break;
		default:
			return FALSE;
	}
	return FALSE;
}

static void PASCAL AppCommand (HWND hwnd, unsigned msg, WPARAM wParam, LPARAM lParam)
{
    int id = GET_WM_COMMAND_ID(wParam, lParam);

    switch(id)
    {
		case BRIGHT_0:
		case BRIGHT_1:
		case BRIGHT_2:
		case BRIGHT_3:
			g_bySelectDecoderNum = id - BRIGHT_0;
			DialogBox (g_hInstance, MAKEINTRESOURCE(IDD_ADJ_COLOR), hwnd, (DLGPROC)DlgProcAdjustColor);
			break;

		case IDM_TEST_OUTPUT:
			DialogBox (g_hInstance, MAKEINTRESOURCE(IDD_OUTPIN), hwnd, (DLGPROC)DlgProcTestInOutPin);
			break;

		case IDM_WDT_START:
			PV_SetWDT(0, START);	//In PV-149/152 WDT card number always 0 or 4.
			SetTimer(ghApp, ID_TIMER_WDT, 1000 * 10, NULL);
			break;
		case IDM_WDT_STOP:
			PV_SetWDT(0, STOP);
			KillTimer(ghApp, ID_TIMER_WDT);
			break;
		case IDM_WDT_TEST:
			PV_SetWDT(0, START);
			KillTimer(ghApp, ID_TIMER_WDT);
			break;
	}

}

LRESULT CALLBACK WndMainProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	HDC         hDC;

    switch (message)
    {
        case WM_GRAPHNOTIFY:
            HandleGraphEvent();
            break;

		case WM_PAINT:
			hDC = BeginPaint(hwnd, &ps);
			InvalidateRect(hwnd, NULL, TRUE);
			EndPaint(hwnd, &ps);
			break;

		case WM_TIMER:
			PV_SetWDT(0, CLEAR);
			break;

		case WM_LBUTTONDOWN:
			BYTE byData;
			PV_GetProtectID(CARD_NUM, &byData);
			Msg(TEXT("S/W Protect ID=0x%x"), byData);
			break;

		case WM_MBUTTONDOWN:
			break;

		case WM_RBUTTONDOWN:
			break;

		case WM_COMMAND:
			AppCommand (hwnd, message, wParam, lParam);
			break;

        case WM_WINDOWPOSCHANGED:
            ChangePreviewState(! (IsIconic(hwnd)));
            break;

        case WM_CLOSE:
            // Hide the main window while the graph is destroyed
            ShowWindow(ghApp, SW_HIDE);
            CloseInterfaces();  // Stop capturing and release interfaces
            break;

        case WM_CREATE:
			BmpInfHdr.biBitCount = 16;
			BmpInfHdr.biClrImportant = 0;
			BmpInfHdr.biClrUsed = 0;
			BmpInfHdr.biCompression = BI_RGB;
			BmpInfHdr.biHeight = CAP_DUP_H;
			BmpInfHdr.biWidth = CAP_DUP_W;
			//BmpInfHdr.biHeight = CAP_DUP_H/2;
			//BmpInfHdr.biWidth = CAP_DUP_W/2;
			BmpInfHdr.biPlanes = 1;
			BmpInfHdr.biSize = sizeof(BITMAPINFOHEADER);
			BmpInfHdr.biSizeImage = CAP_DUP_W * CAP_DUP_H * 2;
			//BmpInfHdr.biSizeImage = (CAP_DUP_W * CAP_DUP_H * 2) /4;
			BmpInfHdr.biXPelsPerMeter = 0;
			BmpInfHdr.biYPelsPerMeter = 0;

			hDrawDib = DrawDibOpen();
			break;

        case WM_DESTROY:
			PV_SetWDT(0, STOP);
			KillTimer(ghApp, ID_TIMER_WDT);

			DrawDibClose(hDrawDib);
            PostQuitMessage(0);
            return 0;
    }
    return DefWindowProc (hwnd , message, wParam, lParam);
}


int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hInstP, LPTSTR lpCmdLine, int nCmdShow)
{
    MSG msg;
    WNDCLASS wc;

    // Initialize COM
    if(FAILED(CoInitialize(NULL)))
    {
        Msg(TEXT("CoInitialize Failed!\r\n"));   
        exit(1);
    } 

    // Register the window class
    ZeroMemory(&wc, sizeof wc);
    wc.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
    wc.lpfnWndProc   = WndMainProc;
    wc.hInstance     = hInstance;
	wc.lpszMenuName   = MAKEINTATOM(IDR_MENU1);
    wc.lpszClassName = CLASSNAME;
    wc.hbrBackground = (HBRUSH)GetStockObject(GRAY_BRUSH);
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hIcon         = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_VIDPREVIEW));
    if(!RegisterClass(&wc))
    {
        Msg(TEXT("RegisterClass Failed! Error=0x%x\r\n"), GetLastError());
        CoUninitialize();
        exit(1);
    }

    // Create the main window.  The WS_CLIPCHILDREN style is required.
    ghApp = CreateWindow(CLASSNAME, APPLICATIONNAME,
                    WS_OVERLAPPEDWINDOW | WS_CAPTION | WS_CLIPCHILDREN,
                    CW_USEDEFAULT, CW_USEDEFAULT,
                    DEFAULT_VIDEO_WIDTH, DEFAULT_VIDEO_HEIGHT,
                    0, 0, hInstance, 0);

    if(ghApp)
    {
        HRESULT hr;
		g_hInstance = hInstance;

		g_VdoWnd_0 = CreateWindow("STATIC",""
						, WS_CHILD| SS_CENTER|WS_VISIBLE,
						0, 0, 320, 240,
						ghApp, NULL, g_hInstance, NULL);
		g_VdoWnd_1 = CreateWindow("STATIC",""
						, WS_CHILD| SS_CENTER|WS_VISIBLE,
						325, 0, 320, 240,
						ghApp, NULL, g_hInstance, NULL);

        // Create DirectShow graph and start capturing video
        hr = CaptureVideo();
        if (FAILED (hr))
        {
            CloseInterfaces();
            DestroyWindow(ghApp);
        }
        else
        {
            // Don't display the main window until the DirectShow
            // preview graph has been created.  Once video data is
            // being received and processed, the window will appear
            // and immediately have useful video data to dispay.
            // Otherwise, it will be black until video data arrives.
            ShowWindow(ghApp, nCmdShow);
        }       

        // Main message loop
        while(GetMessage(&msg,NULL,0,0))
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    }

    // Release COM
    CoUninitialize();

    return msg.wParam;
}
